﻿using UnityEngine;
using System.Collections;

public class GridManager : MonoBehaviour {

    public int rows;
    public int cols;

    public GameObject defaultTile;
    public GameObject waterTile;
    public GameObject castleTile;

    private Tile[,] grid;

	// Use this for initialization
	void Start () {
        grid = new Tile[cols, rows];
        BuildGrid();
	}

    /// <summary>
    /// Build the initial grid with the basic sand tile attached.
    /// </summary>
    private void BuildGrid()
    {
        for (int i = 0; i < rows; i++)
        {
            for (int j=0; j < cols; j++)
            {
                // Build basic sand Tiles here;
                GameObject tmp = Instantiate(defaultTile, new Vector3(j, 0, i), Quaternion.identity) as GameObject;
                grid[j, i] = tmp.GetComponent<Tile>();
                grid[j, i].SetGrid(this, j , i);
            }
        }

        ReplaceTileOffset(castleTile, 0, 3, castleTile.transform.position);
    }


    /// <summary>
    /// Remove all water tiles from the grid
    /// </summary>
    public void ClearWater()
    {
        for (int i=0; i<rows; i++)
        {
            for (int j=0; j<cols; j++)
            {
                // Iterate through grid
                // Check if water tile and remove it if it is.
            }
        }
    }

    /// <summary>
    /// Returns the Tile in the specified spot on the grid
    /// </summary>
    /// <param name="col">The column in the grid to search</param>
    /// <param name="row">The column in the grid to search</param>
    /// <returns></returns>
    public Tile GetTile(int col, int row)
    {
        if (col < cols && row < rows && col >= 0 && row >= 0)
        {
            return grid[col, row];
        }
        else return null;
    }

    /// <summary>
    /// Set a tile location to be a different tile.
    /// Deletes the tile in that location and the associated gameObject.
    /// </summary>
    /// <param name="tile">The tile to replace it with</param>
    /// <param name="col">The column of the tile to replace</param>
    /// <param name="row">The row of the tile to replace</param>
    public GameObject ReplaceTile(GameObject tile, int col, int row)
    {
        if (col < cols && row < rows)
        {
            GameObject tmp = Instantiate(tile, new Vector3(col, 0, row), tile.transform.rotation) as GameObject;
            Destroy(GetTile(col, row).gameObject);
            grid[col, row] = tmp.GetComponent<Tile>();
            tmp.GetComponent<Tile>().SetGrid(this, col, row);
            return tmp;
        }
        return null;
    }

    /// <summary>
    /// Set a tile location to be a different tile.
    /// Deletes the tile in that location and the associated gameObject.
    /// </summary>
    /// <param name="tile">The tile to replace it with</param>
    /// <param name="col">The column of the tile to replace</param>
    /// <param name="row">The row of the tile to replace</param>
    public GameObject ReplaceTileOffset(GameObject tile, int col, int row, Vector3 offset)
    {
        if (col < cols && row < rows)
        {
            GameObject tmp = Instantiate(tile, new Vector3(col, 0, row) + offset, tile.transform.rotation) as GameObject;
            Destroy(GetTile(col, row).gameObject);
            grid[col, row] = tmp.GetComponent<Tile>();
            tmp.GetComponent<Tile>().SetGrid(this, col, row);
            return tmp;
        }
        return null;
    }
}
