﻿using UnityEngine;
using System.Collections;

public class WaterController : Tile {

    private int _dir;
    private System.Timers.Timer _timer;
    private float _timeLeft;

    [SerializeField]
    private GameObject _waterStack;

    public GameObject sandTile;
    public GameObject waterTile;
    public float time;

	/// <summary>
    /// Starts Timer
    /// </summary>
	void Start () {
        _timeLeft = time;
	}
	
	/// <summary>
    /// Update timer, Flow into adjacent areas, Check end game.
    /// </summary>
	void Update () {
        _timeLeft -= Time.deltaTime;

        if (_timeLeft <= 0)
        {
            Flow();
            _timeLeft = time;
            CheckEndGame();
        }        

	}

    /// <summary>
    /// Check if water can hit castle (on col 0 with some strength left)
    /// </summary>
    private void CheckEndGame()
    {
        if ( (GetCol() == 0) && (strength > 1) )
        {
            //Debug.Log("YOU LOST!");
        }
    }

    /// <summary>
    /// Check a tile if it is available for water flow.
    /// </summary>
    /// <param name="col">The column in the grid of tiles.</param>
    /// <param name="row">The row in the grid of tiles.</param>
    /// <returns></returns>
    private bool CheckTile(int col, int row)
    {
        if (row < parentGrid.rows && col < parentGrid.cols && row >= 0 && col >= 0)
            return (parentGrid.GetTile(col, row).IsFlowable());
        else return false;
    }

    /// <summary>
    /// Flow water into the adjacent tiles.
    /// </summary>
    private void Flow()
    {
        int row = GetRow();
        int col = GetCol();
        bool hasChild = false;

        // LEft tile
        if (CheckTile(col - 1, row) && strength > 0)
        {
            _ExpandTo(col - 1, row);
            hasChild = true;
        }
        // above tile
        if (CheckTile(col, row - 1) && strength > 0)
        {
            _ExpandTo(col, row - 1);
            hasChild = true;
        }
        // below tile
        if (CheckTile(col, row + 1) && strength > 0)
        {
            _ExpandTo(col, row + 1);
            hasChild = true;
        }
        //right tile
        if (CheckTile(col + 1, row) && strength > 0)
        {
            _ExpandTo(col + 1, row);
            hasChild = true;
        }        

        if (hasChild == false && strength > 0)
        {
            SetStrength(strength - 1);
        }

        if (strength <= 0)
        {
            parentGrid.ReplaceTile(sandTile, col, row);
        }
    }

    /// <summary>
    /// Set the strength and update the height of water
    /// </summary>
    /// <param name="str">new value of strength</param>
    public new void SetStrength(int str)
    {
        strength = str;

        transform.GetChild(0).position = new Vector3(transform.GetChild(0).position.x, transform.position.y + .5f + (strength * .1f), transform.GetChild(0).position.z);
        transform.GetChild(1).position = new Vector3(transform.GetChild(1).position.x, transform.position.y - .5f + (strength * .1f), transform.GetChild(1).position.z);
    }

    /// <summary>
    /// Water flows into position, losing strength and washing away stuff.
    /// </summary>
    /// <param name="col">col of tile to expand to</param>
    /// <param name="row">row of tile to expand to</param>
    private void _ExpandTo(int col, int row)
    {
        Tile tile = parentGrid.GetTile(col, row);

        while (tile.strength > 1 && strength > 1)
        {
            SetStrength(strength - 1);
            tile.SetStrength(tile.GetStrength() - 1);
            if (tile.GetStrength() <= 1)
            {
                if (Random.Range(0.0f, 1.0f) < .2 && string.Compare(tile.GetTileType(), "trench") == 0)
                {
                    parentGrid.ReplaceTile(UpgradesTo, col, row);
                    return;
                }

                GameObject obj = parentGrid.ReplaceTileOffset(tile.DowngradesTo, col, row, new Vector3(0, tile.transform.position.y, 0));
                tile = obj.GetComponent<Tile>();
                break;
            }
        }
        
        if (tile.strength <= 1)
        {
            SetStrength(strength - 1);
            GameObject tmp = parentGrid.ReplaceTile(waterTile.GetComponent<WaterPrefabHolder>().GetPrefab(), col, row);
            tmp.GetComponent<WaterController>().SetStrength(strength);            


        }
        
    }
}
